######################################################################
## Analysis of DFA reproducibility results
######################################################################
## Written by the Reproducibility Group: Rose Andrew, Arianne Albert, Sebastien Renaut
## Diana Rennison, Dan Bock and Tim Vines
##
## This script analyses the output of our reanalyes to produce the results
## presented in our paper.It summarises the results and conducts linear models, GLMs,
## Fisher's exact tests to examine the effects of publication year and software on
## the outcomes of the reanalysis. The code also generates the plots and tables
## shown in the paper.
##
## The input file is contains the study ID, the publication year, the software used
## in the original paper, the values of PVE, a coefficient and PAC from the original paper 
## and the corresponding results from our own reanalysis. Also in this file, studies
## that were excluded initially or could not be reanalysed are identified and reasons given.

## Note: this script is intended to be run line-by-line
########################################################################

########################################################################
## Read results and set up factors
######################################################################

## Read data file - need to specify quotes because some fields contain apostrophes
dfa_repro = read.table("Reanalysis_output_public.txt", sep = "\t", na.strings = c("NA", "", ".", "#DIV/0!", "#REF!", 
    "#VALUE!", "#NUM!", "9999"), header = T, quote = "\"", blank.lines.skip = TRUE, stringsAsFactors = F)
npapers = nrow(dfa_repro)

## Summarise Category and Software
table(dfa_repro$Category)
dfa_repro$Category = factor(dfa_repro$Category)
# D: Data discrepancy
# E: Excluded prior to study
# I: Incorrect data file
# M: Insufficient metadata
# R: Reanalysed

dfa_repro$Software.used[dfa_repro$Software.used %in% c("STATGRAPHICS", "LINDA", "S-PLUS")] = "Other"
table(dfa_repro$Software.used)

## Data problem - include Incorrect data file, Insufficient metadata and Data discrepancy
dfa_repro$data.problem = dfa_repro$Category %in% c("I", "M", "D")

## Omit studies that were excluded prior to the study
include = dfa_repro$Category != "E"
table(dfa_repro$Category[include])

##########################################################
## Effect of publication year (PY) on Category
##########################################################

## Data problem - either incorrect file, poor labeling or transformation needed
# Insufficient metadata -> Results para 3
glm.metadata = glm(Category == "M" ~ PY, family = binomial, data = dfa_repro[include, ])
summary(glm.metadata)
drop1(glm.metadata, test = "Chisq")
exp(coef(glm.metadata))
exp(confint.default(glm.metadata))
## odds ration 0.98, P = 0.82


# Data discrepancy -> Results para 3
glm.discrepancy = glm(Category == "D" ~ PY, family = binomial, data = dfa_repro[include, ])
summary(glm.discrepancy)
drop1(glm.discrepancy, test = "Chisq")
exp(coef(glm.discrepancy))
exp(confint.default(glm.discrepancy))
## odds ration 1.06, P = 0.45


## -> Results para 4
glm.data.problem = glm(data.problem ~ PY, family = binomial, data = dfa_repro[include, ])  # excluding methods problems
summary(glm.data.problem)
drop1(glm.data.problem, test = "Chisq")
exp(coef(glm.data.problem))
exp(confint.default(glm.data.problem))
## odds ration 1.04, P = 0.46


#####################################################
## Effect of statistical software on category
#####################################################
##Data problems 
## Only where stated -> Results para 4
dfa_repro1 = dfa_repro[which(include & !dfa_repro$Software.used %in% c("Not stated")), ]
fisher.test(table(dfa_repro1$data.problem, dfa_repro1$Software.used))
## p-value = 0.01223

## Counts of each category by software
t1 = table(dfa_repro1$data.problem, dfa_repro1$Software.used)
t1

# SAS proportion
t1["TRUE", "SAS"]/sum(t1[, "SAS"])

# Overall proportion
table(dfa_repro$data.problem[include])[2]/sum(table(dfa_repro$data.problem[include]))


#####################################################
## Correlation between published and reanalysed summary statistics -> Results para 4
#####################################################
cor.test(dfa_repro$PVE_paper, dfa_repro$PVE_best, na.action = "na.omit")
# cor=0.94
cor.test(dfa_repro$PAC_paper, dfa_repro$PAC_best, na.action = "na.omit")
# cor=0.95
cor.test(abs(dfa_repro$COEF_paper), abs(dfa_repro$COEF_best), na.action = "na.omit")
# cor=0.96


#####################################################
## Match (within 1% of the published value) or close (within 5%?
#####################################################

## Calculate unsigned difference between published and reanalysed values
dfa_repro$PVE_diff_perc = 100 * (abs(dfa_repro$PVE_best - dfa_repro$PVE_paper)/dfa_repro$PVE_paper)
dfa_repro$COEF_diff_perc = 100 * (abs(abs(dfa_repro$COEF_best) - abs(dfa_repro$COEF_paper))/abs(dfa_repro$COEF_paper))
dfa_repro$PAC_diff_perc = 100 * (abs(dfa_repro$PAC_best - dfa_repro$PAC_paper)/dfa_repro$PAC_paper)


## PVE -> Results para 4
PVE_match = dfa_repro$PVE_diff_perc <= 1
PVE_close = dfa_repro$PVE_diff_perc <= 5
nPVE = length(which(!is.na(PVE_match)))
nPVE
table(PVE_match)
table(PVE_match)/nPVE
table(PVE_close)
table(PVE_close)/nPVE
## 26 total; 20 within 1% (80%); 24 within 5%(92%)

## PAC
PAC_match = dfa_repro$PAC_diff_perc <= 1
PAC_close = dfa_repro$PAC_diff_perc <= 5
nPAC = length(which(!is.na(PAC_match)))
nPAC
table(PAC_match)
table(PAC_match)/nPAC
table(PAC_close)
table(PAC_close)/nPAC
## 59 total; 44 within 1% (73%); 56 within 5%(93%)


## COEF
COEF_match = dfa_repro$COEF_diff_perc <= 1
COEF_close = dfa_repro$COEF_diff_perc <= 5
nCOEF = length(which(!is.na(COEF_match)))
nCOEF
table(COEF_match)
table(COEF_match)/nCOEF
table(COEF_close)
table(COEF_close)/nCOEF
## 26 total; 15 within 1% (58%); 15 within 5%(58%)

## All statistics combined - results para 5
total = nPVE + nPAC + nCOEF
total_match = table(PVE_match) + table(PAC_match) + table(COEF_match)
total_close = table(PVE_close) + table(PAC_close) + table(COEF_close)
total_match
total_match/total
total_close
total_close/total
## 112 total; 79 within 1% (71%); 95 within 5%(85%)


## Summarize matches - results para 5
##  Function for summarising into "Complete match", "Partial match" and "No match"
categ = function(x, df) {
    if (all(is.na(df[x, ]))) 
        NA else if (any(!df[x, ], na.rm = T)) {
        if (any(df[x, ], na.rm = T)) 
            "Partial match" else "No match"
    } else if (any(df[x, ], na.rm = T)) 
        "Complete match"
}

## Within 5% of the published value
res_close_frame = cbind(PVE_close, COEF_close, PAC_close)
dfa_repro$res_close = sapply(1:npapers, categ, res_close_frame)
# Check that res_close is correct
cbind(res_close_frame, dfa_repro$res_close)

## Within 1% of the published value
res_match_frame = cbind(PVE_match, COEF_match, PAC_match)
dfa_repro$res_match = sapply(1:npapers, categ, res_match_frame)

## Summarise
table(dfa_repro$res_match)
table(dfa_repro$res_match)/length(which(!is.na(dfa_repro$res_match)))
table(dfa_repro$res_close)
table(dfa_repro$res_close)/length(which(!is.na(dfa_repro$res_close)))

#####################################################
## Linear models of the difference vs year - results para 6
#####################################################
## PVE
## by year -> Results para 5
# raw - n.s.
lm_PVE_diff = lm(PVE_diff_perc ~ PY, na.action = na.omit, data = dfa_repro)
summary(lm_PVE_diff)
drop1(lm_PVE_diff, test = "F")
# transformed - n.s.
lm_PVE_diff = lm(asin(sqrt(PVE_diff_perc/100)) ~ PY, na.action = na.omit, data = dfa_repro)
summary(lm_PVE_diff)
drop1(lm_PVE_diff, test = "F")

## by software
lm_PVE_diff = lm(PVE_diff_perc ~ Software.used, na.action = na.omit, data = dfa_repro)
summary(lm_PVE_diff)  # R seems high
drop1(lm_PVE_diff, test = "F")  # significant for software, but driven by a single study
table(!is.na(dfa_repro$PVE_paper), dfa_repro$Software.used)
summary(update(lm_PVE_diff, data = dfa_repro[dfa_repro$Software.used != "R", ]))


## COEF 
## by year -> Results para 5
# transformed - n.s.
lm_COEF_diff = lm(log(COEF_diff_perc) ~ PY, na.action = na.omit, data = dfa_repro)
summary(lm_COEF_diff)
# raw - n.s.
lm_COEF_diff = lm(COEF_diff_perc ~ PY, na.action = na.omit, data = dfa_repro)
summary(lm_COEF_diff)

## by software - n.s.
lm_COEF_diff = lm(COEF_diff_perc ~ Software.used, na.action = na.omit, data = dfa_repro)
summary(lm_COEF_diff)
drop1(lm_COEF_diff, test = "F")  # n.s.


## PAC
## by year -> Results para 5
# transformed - n.s.
lm_PAC_diff = lm(asin(sqrt(PAC_diff_perc/100)) ~ PY, na.action = na.omit, data = dfa_repro)
summary(lm_PAC_diff)
# linear - n.s
lm_PAC_diff = lm(PAC_diff_perc ~ PY, na.action = na.omit, data = dfa_repro)
summary(lm_PAC_diff)


## Software effect -> Results para 5 - n.s.
lm_PAC_diff = lm((PAC_diff_perc) ~ Software.used, na.action = na.omit, data = dfa_repro)
summary(lm_PAC_diff)
drop1(lm_PAC_diff, test = "F")


## Analysis of software effect on result (whether values match within 1% or 5%)
# Result given no data problems: not significant
table(dfa_repro$Software.used[include & !dfa_repro$data.problem])
fisher.test(table(dfa_repro$res_match[include & !dfa_repro$data.problem & !dfa_repro$Software.used %in% c("Not stated", 
    "Other", "MATLAB", "JMP")], dfa_repro$Software.used[include & !dfa_repro$data.problem & !dfa_repro$Software.used %in% 
    c("Not stated", "Other", "MATLAB", "JMP")]))
fisher.test(table(dfa_repro$res_close[include & !dfa_repro$data.problem & !dfa_repro$Software.used %in% c("Not stated", 
    "Other", "MATLAB", "JMP")], dfa_repro$Software.used[include & !dfa_repro$data.problem & !dfa_repro$Software.used %in% 
    c("Not stated", "Other", "MATLAB", "JMP")]))

# Result given no data problems, excluding small groups (R) - n.s.
table(dfa_repro$Software.used[include & !dfa_repro$data.problem])
fisher.test(table(dfa_repro$res_match[include & !dfa_repro$data.problem & !dfa_repro$Software.used %in% c("Not stated", 
    "Other", "MATLAB", "JMP", "R")], dfa_repro$Software.used[include & !dfa_repro$data.problem & !dfa_repro$Software.used %in% 
    c("Not stated", "Other", "MATLAB", "JMP", "R")]))


#######################################################################
## Plotting figures 2,3,4
#######################################################################

### FIGURE 2
par(mar = c(5, 5, 2, 2))
plot(dfa_repro[, c("PVE_paper", "PVE_best")], cex = 1.3, pch = 16, ylab = "Reanalysed PVE", xlab = "Published PVE", 
    cex.lab = 1.3)
lines(c(40, 100), c(40, 100))

dev.print(device = pdf, "Figure2_PVE.pdf", onefile = F)
dev.off()


### FIGURE 3
par(mar = c(5, 5, 2, 2))
plot(dfa_repro[, c("PAC_paper", "PAC_best")], cex = 1.3, pch = 16, ylab = "Reanalysed PAC", xlab = "Published PAC", 
     cex.lab = 1.3,xlim=c(38,100),ylim=c(38,100))
lines(c(38, 100), c(38, 100))

dev.print(device = pdf, "Figure3_PAC.pdf", onefile = F)
dev.off()


### FIGURE 4
par(mar = c(5, 5, 2, 2))
plot(abs(dfa_repro[, "COEF_paper"]), abs(dfa_repro[, "COEF_best"]), cex = 1.3, pch = 16, ylab = "Reanalysed Coefficient (absolute value)", 
    xlab = "Published Coefficient (absolute value)", xlim = c(0, 20.5), ylim = c(0, 20.5), cex.lab = 1.3)
lines(c(0, 20), c(0, 20))

dev.print(device = pdf, "Figure4_coefficient.pdf", onefile = F)
dev.off()


#######################################################################
## Making tables tables 1 and 2
#######################################################################

## Make summary table (Table 1)
tab1 = data.frame(Software = levels(factor(dfa_repro$Software.used)))

tab1$Software = as.character(tab1$Software)
tab1$Software[tab1$Software == "Not stated"] = "unknown"
tab1$Excluded = table(factor(dfa_repro$Software.used)[which(dfa_repro$Category == "E")])
tab1$Included = table(factor(dfa_repro$Software.used)[which(dfa_repro$Category != "E")])
tab2 = table(dfa_repro$Software.used, dfa_repro$Category == "I")[, 2]
tab3 = table(dfa_repro$Software.used, dfa_repro$Category == "M")[, 2]
tab4 = table(dfa_repro$Software.used, dfa_repro$Category == "D")[, 2]
tab6 = table(dfa_repro$Software.used, dfa_repro$res_match)[, c(2, 3, 1)]
tab1 = cbind(tab1, tab2, tab3, tab4, tab6)
names(tab1) = c("Software", "Excluded", "Included", "Incorrect data file", "Insufficient metadata", "Data discrepancy", 
    "No match", "Partial match", "Complete match")
tab1 = tab1[c(1, 2, 5:9, 4, 3), ]
## Percentages
tab1_percent = tab1
tab1_percent[, 4:9] = round(100 * tab1[, 4:9]/tab1$Included)
apply(tab1_percent[, 4:9], 1, sum)  # some sum to 99 or 101 because of rounding

write.table(cbind(tab1, tab1_percent), file = "Summary_table.txt", row.names = F, col.names = T, quote = F, sep = "\t")


## Make detailed table (Table 2)
outfile = dfa_repro[, c("Study.no.", "PY", "Software.used", "PVE_paper", "PVE_best", "COEF_paper", "COEF_best", 
    "PAC_paper", "PAC_best", "Category", "Reason", "res_match", "res_close")]

names(outfile) = c("Study no.", "Publication year", "Software used", "PVE published", "PVE reanalyzed", "COEF published", 
    "COEF reanalyzed", "PAC published", "PAC reanalyzed", "Category", "Reason", "Match (within 1%)", "Close (within 5%)")
outfile$"PVE published" = round(outfile$"PVE published", 1)
outfile$"PVE reanalyzed" = round(outfile$"PVE reanalyzed", 1)
outfile$"COEF published" = round(outfile$"COEF published", 3)
outfile$"COEF reanalyzed" = round(outfile$"COEF reanalyzed", 3)
outfile$"PAC published" = round(outfile$"PAC published", 1)
outfile$"PAC reanalyzed" = round(outfile$"PAC reanalyzed", 1)
outfile$Reason[outfile$Category %in% c("Complete match", "Partial match", "No match")] = ""
outfile[, "Match (within 1%)"] = substr(dfa_repro$res_match, 1, 1)
outfile[, "Close (within 5%)"] = substr(dfa_repro$res_close, 1, 1)
outfile[is.na(outfile)] = ""

write.csv(outfile, file = "Reanalysis_results.csv", row.names = F)


## Also write all columns to file
write.csv(dfa_repro, file = "Reanalysis_all_columns.csv", row.names = F) 


#######################################################################
## Output from sessionInfo()
#######################################################################
#R version 3.2.0 (2015-04-16)
#Platform: x86_64-w64-mingw32/x64 (64-bit)
#Running under: Windows 7 x64 (build 7601) Service Pack 1
#
#locale:
#  [1] LC_COLLATE=English_Australia.1252  LC_CTYPE=English_Australia.1252    LC_MONETARY=English_Australia.1252
#[4] LC_NUMERIC=C                       LC_TIME=English_Australia.1252    
#
#attached base packages:
#  [1] stats     graphics  grDevices utils     datasets  methods   base     
#
#loaded via a namespace (and not attached):
#  [1] tools_3.2.0
